package Sophos::SAVI::Engine;

use strict;
use base qw(DynaLoader Exporter);

our $VERSION = '1.000';
our @EXPORT_OK = qw(
    $NOTIFY_CONTINUE_THIS
    $NOTIFY_CONTINUE_NEXT
    $NOTIFY_STOP
    $NOTIFY_DEFAULT
    $ACTIVITY_CLASSIFY
    $ACTIVITY_NEXTFILE
    $ACTIVITY_DECOMPRESS
);

Sophos::SAVI::Engine->bootstrap($VERSION);

sub SAVI_ENTRY_POINT() { "DllGetClassObject" }

sub _dl_load_libsavi {
    my ($old_libref, $new_libsavi) = @_;

    # If this perl doesn't support unloading shared libraries, return the
    # symbol from the currently-loaded library.
    if ($old_libref) {
	if (!defined &DynaLoader::dl_unload_file) {
	    my $sym = DynaLoader::dl_find_symbol($old_libref, SAVI_ENTRY_POINT);
	    return ($old_libref, $sym);
	}
	DynaLoader::dl_unload_file($old_libref);
    }
    my $libref = DynaLoader::dl_load_file($new_libsavi, 0);
    my $symref = DynaLoader::dl_find_symbol($libref, SAVI_ENTRY_POINT);
    return ($libref, $symref);
}

1;

__END__

=head1 NAME

Sophos::SAVI::Engine - interface to the Sophos virus
scanning library.

=head1 SYNOPSIS

   use Sophos::SAVI::Engine;
   my $scanner = Sophos::SAVI::Engine->new(
       $datadir,
       scan_macros => 1,
       unzip => 1,
       untar => 1,
       ungzip => 1,
   ) or die "can't create new Sophos::SAVI::Engine object.";

   my $results = $scanner->scan_file("/tmp/klez.exe");
   for my $file (keys %$results) {
       next unless $file->{infected};
       my @virii = @{$file->{viruses}};
   }

=head1 DESCRIPTION

This module exposes the Sophos Anti-Virus Interface (SAVI) to Perl. It is
intended to support using Sophos as a plugin to the Sophos-SAVI package.

=head2 new()

   $obj = Sophos::SAVI::Engine->new($datadir, %options);

Creates a new object which can be used to can files for viruses. Returns
C<undef> if an error occurs while initialising the library.

The C<$datadir> argument is required.

=over 4

=item logger

If specified, an object which implements a C<log()> method like this:

   $obj->log("SEVERITY", "message");

The engine holds a reference to the object. Currently the only message
generated is when the engine detects that the DAT files have been updated, and
reloads itself.

=item notify

If specified, a hash reference with the following structure:

  notify => {
    OnFileFound => \&onFileFound,
    OnVirusFound => \&onVirusFound,
    OnErrorFound => \&onErrorFound,
    OnClassification => \&onClassification,
    OkToContinue => \&okToContinue,
  },
  ...

The SAVI engine will invoke the callbacks during the scan operation. Note that
providing these callbacks may affect the scanning speed.

Callbacks are passed the following parameters:

=over

=item OnFileFound

  sub onfilefound {
    my $filename = shift;
  }

The return value should be one of the callback return values (see L</Callback
Return Values>).

=item OnVirusFound

  sub onvirusfound {
    my @ids = @_;
  }

The return value should be one of the callback return values (see L</Callback
Return Values>).

=item OnErrorFound

  sub onerrorfound {
    my ($errorcode, $errorstring) = @_;
  }

The return value should be one of the callback return values (see L</Callback
Return Values>).

=item OnClassification

  sub onclassification {
    my ($typecode, $typestring) = @_;
  }

The return value should be one of the callback return values (see L</Callback
Return Values>).

I<NOTE>: as the Sophos Engine develops, new type codes are added that may not
have type strings. In this case, the C<$typestring> parameter will be
C<undef>.

=item OkToContinue

  sub oktocontinue {
    my ($activity, $extent, $target) = @_;
  }

The return value should be one of the callback return values (see L</Callback
Return Values>).

Possible values for the C<$activity> parameter are listed in L</Activity
Codes>.

=back

=back

Please see L</Sophos Options> for a list of options, their meanings and
defaults.

=head2 scan_file()

   $results = $obj->scan_file(@files);

Scans files for viruses. Returns a data structure that looks like this:

   {
       'file1' => [],
       'file2' => ['virus1.id', 'virus2.id'],
   }

If you passed one of the un* options, files inside an archive or compressed
file are represented as files in a directory:

   {
       'file.zip' => [],
       'file.zip/hello.exe' => ['virus.id'],
   }

=head2 clean_file()

   $result = $obj->clean_file(@files);

Scans files and attempts to clean all viruses.  Returns a data structure
that looks like this:

   {
       'file1' => 'cleaned',
       'file2' => 'cleanfailed',
   }

=head1 Sophos Options

You can specify any of the following options to the constructor by passing a
list of key/value pairs.

The following options enable or disable multiple options:

=over 4

=item GrpArchiveUnpack

All archive and compressed archive file formats (e.g. ZIP, UUE, etc).

=item GrpSelfExtract

File formats that contain an executable stub that automatically decompresses
the body of the file.

=item GrpExecutable

Executable files.

=item GrpInternet

File formats commonly in use on the internet.

=item GrpMSOffice

Microsoft Office file formats.

=item GrpMisc

File formats that do not fall into any of the above categories.

=item GrpDisinfect

Enables or disables disinfection of all files for which disinfection is
supported.

=item GrpClean

All "clean" file formats.

=item GrpSuper

Any option that is part of a group is also included in this group.

=back

The following options correspond to the Sophos configuration options. Where
applicable, the option group that contains the option is mentioned.

=over 4

=item Access

Enables scanning of databases saved in Microsoft Access (MDB) format.

Group: GrpMSOffice

Default: off

=item ActiveMimeHandling

Enables the detection of macro viruses within Microsoft Office documents saved
in ActiveMime document format.

Group: GrpMSOffice

Default: on

=item AppleSingle

Enables handling of Macintosh files saved as a single file in situations where
support for Macintosh dual forks is not possible (e.g. MIME files).

Group: GrpArchiveUnpack

Default: off

=item ArjDecompression

Enables decompression of Arj archives.

Group: GrpArchiveUnpack

Default: off

=item Bzip2

Enables decompression of BZip2 archives.

Group: GrpArchiveUnpack

Default: off

=item CleanBmp

=item CleanGif

=item CleanJpeg

=item CleanLnk

=item CleanPng

=item CleanRiff

=item CleanTiff

Certain file formats (e.g. .bmp bitmap files) cannot contain viruses. These
options enable files of the corresponding types to be positively identified
and scanning of them to be stopped. This results in more efficient scanning
when handling these file types.

Group: GrpClean

Default: on

=item CmzDecompression

Enables decompression of Cmz archives.

Group: GrpArchiveUnpack

Default: off

=item DecompressBVA5

Decompresses the text part of a VBA5 project before scanning. This area is
often corrupted, but can help positive identification of some viruses.

Group: GrpMSOffice

Default: on

=item DelVBA5Project

Causes the entire project containing any viral macros to be removed during
disinfection.

Default: on

=item DynamicDecompression

Controls the ability to scan inside supported types of dynamically compressed
executables. This type of file appears to be a normal executable. The main
part of the program is compressed and extracted into memory at run-time.
Sophos recommends this option should be turned on.

Group: GrpSelfExtract

Default: on

=item Elf

Controls the module that interprets ELF binary files. This format is used for
many Linux and FreeBSD executable files.

Group: GrpExecutable

Default: on

=item Emulation

Enables the 16-bit x86 emulation engine, which assists in the detection of
polymorphic executable file viruses. Sophos recommends this option should be
turned on.

Group: GrpExecutable

Default: on

=item ExcelFormulaHandling

Enables scanning of Excel formulas for known formula viruses. Formulas occupy
cells on the worksheet and should not be confused with macros.

Group: GrpMSOffice

Default: on

=item ExecFileDisinfection

Some executable file viruses can be removed from files fairly safely. This
option enables that process.

Group: GrpDisinfect

Default: on

=item FullMacroSweep

Enables a fallback mechanism that has been deployed in the past when new file
structures are encountered. Sophos recommends that you keep this option turned
off.

Default: off

=item FullPdf

Enables every part of a PDF file to be scanned, not just the embedded file
streams. This takes longer.

Default: off

=item FullSweep

Enables full sweeping, which performs extensive fixed pattern matching as well
as the standard intelligent virus search. Turning on this option may have
severe performance implications.

Default: off

=item GZipDecompression

Enables decompression of Gzipped archives.

Group: GrpArchiveUnpack

Default: off

=item HelpHandling

Enables checking for files embedded in Microsoft Help files.

Group: GrpMSOffice

Default: on

=item HqxDecompression

Controls the ability to extract files from BinHex archives.

Group: GrpArchiveUnpack

Default: off

=item Html

Controls the module that optimises scanning in HTML files.

Group: GrpInternet

Default: on

=item IgnoreTemplateBit

Microsoft Office documents that contain executable macros will usually have a
single bit in the file header set to 1 to indicate the presence of executable
code. Under normal circumstances macros contained in a file that does not have
this bit set are not able to execute. If this option is turned off, the Virus
Engine will scan only files with this template bit set to 1. Sophos recommends
that you keep this option turned on.

Default: on

=item ISCabinet

Enables decompression of Cabinet files created using the InstallShield
compression format (e.g. _sys1.cab).

Group: GrpArchiveUnpack

Default: off

=item ISCabinetFull

Increases detection reliability in old format InstallShield files, but with a
slight performance overhead.

Group: GrpArchiveUnpack

Default: on

=item ITSS

Enables scanning of Microsoft Compressed Help files (.chm, .hxs).

Group: GrpMisc

Default: off

=item Java

Enables scanning of Java class files.

Group: GrpExecutable, GrpInternet

Default: on

=item Lha

Enables decompression of Lha archives.

Group: GrpArchiveUnpack

Default: off

=item LoopBackEnabled

Controls SAVI's ability to scan inside certain types of container file. Sophos
recommends that this option should be turned on.

Group: GrpMisc

Default: off

=item MachO

Enables scanning of Mach-O executables (used on Mac OS X). Note that O is the
letter, not a zero.

Group: GrpExecutable

Default: on

=item MaxRecursionDepth

Controls the maximum number of times the engine will recurse into archives
when they are found (e.g. when a zip file is compressed within another zip
file). The maximum value is 100, but SAVI internally has some extra levels of
recursion to cover internal data streams found within some file types.

Default: 16

=item MbinDecompression

Controls the ability to look inside MacBinary archives.

Default: off

=item Mime

Enables decoding and scanning inside a MIME encoded message block.

Group: GrpInternet

Default: off

=item MSCabinet

Enables decompression of Microsoft Cabinet files (cab). (Partial files at the
beginning and end of multi-part cabinet files will not be fully scanned).

Default: off

=item MsCompress

Controls the ability to scan inside files compressed with the Microsoft
compression utility.

Group: GrpArchiveUnpack

Default: off

=item NamespaceSupport

Controls handling of Macintosh (resource + data fork) files and archives (on
platforms where these are supported). Settings are SOPHOS_MAC_FILES (enable
scanning for Mac viruses), SOPHOS_DOS_AND_MAC_FILES (also scan data fork) or
SOPHOS_DOS_FILES (disable Mac file scanning).

Default: off

=item Office2001Handling

Controls the ability of the engine to understand Microsoft Office 2001 files.
This version of Office was only released on the Macintosh platform.

Group: GrpMSOffice

Default: on

=item OF95DecryptHandling

Password-protected Office 95 documents are encrypted with a very simple
algorithm.  Enabling this option causes the virus engine to break this
encryption to search for viruses.

Group: GrpMSOffice

Default: off

=item OleDataMsoHandling

Enables handling of files saved in HTML format from Microsoft Office.

Group: GrpMSOffice

Default: on

=item OleRawHandling

Enables scanning of other OLE types (e.g.  messages saved in Microsoft
Outlook).

Group: GrpInternet, GrpMSOffice

Default: on

=item OleScriptHandling

Enables scanning of Access macros.

Group: GrpMSOffice

Default: on

=item Ole2FileDisinfection

This option enables disinfection of OLE2 (Microsoft Office) files to be
configured independently of other types of file disinfection.

Group: GrpDisinfect

Default: on

=item Ole2Handling

Enables the intelligent scanning of OLE2 (Microsoft Office) format documents.
Sophos recommends that this option should be turned on.

Group: GrpMSOffice

Default: on

=item OpenMacRf

Enables detection and opening of Mac resource forks in all file types scanned.

Default: on

=item OutlookExpress

Enables scanning of files contained within Outlook Express mailboxes. NB: to
scan email attachments, set the Mime option.

Group: GrpInternet

Default: off

NOTE: this option corresponds to the C<Outlook Express> option in libsavi, but
without the space.

=item PalmPilotHandling

Enables the scanning of Palm Pilot format files (.prc).

Group: GrpMisc

Default: on

=item Pdf

Controls the PDF file interpretation module.

Default: GrpMisc

Default: on

=item PeEmulator

Enables the 32 bit executable emulator. This is required to support some
advanced virus detection modes in 32 bit executable files.

Default: GrpExecutable

Default: on

=item PEHandling

Enables the intelligent scanning of Win32/PE format files. This is the
standard format for Windows executable binaries such as .exe and .dll.

Group: GrpExecutable

Default: on

=item PowerPointEmbedded

Enables the scanning of files embedded within Microsoft PowerPoint
presentations.

Group: GrpMSOffice

Default: on

=item PowerPointMacroHandling

Enables the scanning of macros in PowerPoint presentations.

Group: GrpMSOffice

Default: on

=item ProjectHandling

Enables the scanning of VBA macros in Microsoft Project.

Group: GrpMSOffice

Default: on

=item RarDecompression

Enables decompression of RAR archives.

Group: GrpArchiveUnpack

Default: off

=item Rpm

Enables the scanning of distribution archives in RedHat Package Manager
format. NB: to scan .rpm files, set the GZipDecompression and UnixArchive
options as well.

Group: GrpArchiveUnpack

Default: off

=item Rtf

Enables scanning for objects that may be embedded inside Rich Text Format
files.

Group: GrpMSOffice, GrpMisc

Default: on

=item ScrapObjectHandling

Enables the handling of scrap objects that are used in the transfer of data
between certain applications.

Group: GrpMSOffice

Default: on

=item SfxArchives

Enables the virus engine to search in supported types of self-extracting
archive.

Group: GrpArchiveUnpack, GrpSelfExtract

Default: off

=item SrpStreamHandling

Deals with another type of data within Office documents.

Group: GrpMSOffice

Default: on

=item StorageDetOnly

Virus detection has two stages. First a file is identified and classified
according to its type.  In the case of archive files, the individual files
within the archive are extracted and treated in the same way. Then scanning is
then carried out according to the file type.  When StorageDetOnly is enabled,
virus scanning is not carried out. The virus engine simply identifies the file
types and returns information about any files contained inside.

Default: off

=item Stuffit

Enables decompression of Stuffit archives.

Group: GrpArchiveUnpack

Default: off

=item TarDecompression

Enables scanning inside Unix tar archives.

Group: GrpArchiveUnpack

Default: off

=item TnefAttachmentHandling

Enables decoding of files encoded with the Microsoft TNEF format, used by some
mail client applications.

Group: GrpArchiveUnpack

Default: off

=item UnixArchive

Enables scanning inside Unix archive types (ar, cpio).

Group: GrpArchiveUnpack

Default: off

=item Upx

Enables scanning inside UPX-compressed self-extracting executable archives.

Group: GrpSelfExtract

Default: off

=item UTF16

Enables scanning of web pages created using UTF16 (Unicode) character
encoding.

Group: GrpInternet

Default: off

=item UueDecompression

Enables the virus engine to decode UUEncoded files. The UUEncode portion must
start within the first 4k of the document.

Group: GrpArchiveUnpack

Default: off

=item VBA3Handling

=item VBA5Handling

These two options enable the scanning of various versions of Microsoft Visual
Basic for Applications embedded in supported file types, like Office and Visio
documents.

Group: GrpMSOffice

Default: on

=item Vba5p

Enables detection of macros in p-code (compiled Visual Basic) as well as in
Visual Basic source code.

Group: GrpMSOffice

Default: off

=item Vbe

Enables the scanning of encoded VBScript files.

Group: GrpInternet

Default: on

=item VbFiltering

Enables canonicalisation of Visual Basic code. This increases the detection
efficiency of the other Visual Basic for Applications (VBA) plugins.

Group: GrpInternet

Default: off

=item VisioFileHandling

Enables the scanning of Microsoft Visio files.

Group: GrpMSOffice

Default: on

=item WordB

Enables the scanning of WordBasic.

Group: GrpMSOffice

Default: on

=item ZipDecompression

Enables scanning inside Zip archives.

Group: GrpArchiveUnpack

Default: off

=item ZipUseChd

Improves reliability of scanning of Zip files created in a non-standard way
(local file directory invalid).

Group: GrpArchiveUnpack

Default: on

=item EnableAutoStop

Stops the scanner when a potential resource (e.g. disk or memory) consumption
condition is detected (possibly a Denial of Service attack).  This option
uses a set of heuristic tests so it is possible to get the occasional false
positive.

Default: off

=back

The following options are supported for backwards compatibility:

=over 4

=item fullsweep

See C<FullSweep>.

=item emulation

See C<Emulation>.

=item scan_macros

See C<FullMacroSweep>.

=item scan_templates

See C<IgnoreTemplateBit>.

=item scan_pe

See C<PEHandling>.

=item scan_xls

See C<ExcelFormulaHandling>.

=item scan_ppt

See C<PowerPointMacroHandling>.

=item unzipexe

See C<DynamicDecompression>.

=item ungzip

See C<GZipDecompression>.

=item unarj

See C<ArjDecompression>.

=item uncmz

See C<CmzDecompression>.

=item unrar

See C<RarDecompression>.

=item untar

See C<TarDecompression>.

=item unuue

See C<UueDecompression>.

=item unzip

See C<ZipDecompression>.

=item maxrecurse

See C<MaxRecursionDepth>.

=back

=head1 Callback Return Values

The following constants can be imported from
Sophos::SAVI::Engine:

=over

=item $NOTIFY_CONTINUE_THIS

Go ahead and scan this sub-file.

=item $NOTIFY_CONTINUE_NEXT

Skip this sub-file, but go on to the next sub-file.

=item $NOTIFY_STOP

Perform no further scanning on this file.

=item $NOTIFY_DEFAULT

Adopt the default behaviour for this callback.

=back

=head1 Activity Codes

The following constants are passwd to the C<OkToContinue> callback:

=over

=item $ACTIVITY_CLASSIFY

SAVI is matching file contents against virus patterns.

=item $ACTIVITY_NEXTFILE

SAVI has found another sub-file in the file being scanned.

=item $ACTIVITY_DECOMPRESS

SAVI is expanding a compressed file or sub-file.

=back

=head1 COPYRIGHT

Copyright (c) 2002-2005, ActiveState Corporation. All Rights Reserved.

Based on example code provided by Sophos Plc with SAVI. Restrictions may
apply.

=cut
